// Rules parsing
// antecedent
function getAntecedent(rule) {
    let withoutIf = rule.split('if');
    return withoutIf[1].trim().split('then')[0].trim();
}

// TODO: remove duplicate
function getAntecedentComponents(antecedent) {
    return antecedent.split('and').map((i) => i.trim());
}

// consequent
function getConsequent(rule) {
    let withoutIf = rule.split('if');
    return withoutIf[1].trim().split('then')[1].trim();
}

// TODO: remove duplicate
function getConsequentComponents(consequent) {
    return consequent.split('and').map((i) => i.trim());
}

// common
function getVariable(variableComponents) {
    return variableComponents.split('is')[0].trim();
}

function getVariableValue(variableComponents) {
    return variableComponents.split('is')[1].trim();
}

// Rules creation

/* exported MessageTypesEnum */
let MessageTypesEnum = {
    INFO: "info",
    SUCCESS: "success",
    WARNING: "warning",
    DANGER: "danger"
};
Object.freeze(MessageTypesEnum);

function isEmpty(value) {
    if (typeof value === "function") {
        return false;
    }
    return (value == null || value.length === 0);
}

/* exported showFeedbackMessage */
function showFeedbackMessage(message, type) {
    alert(type + ' ' + message);
}

/* exported errorHandler */
function errorHandler(response, callBack, errorCallBack) {
    if (!isEmpty(response.error)) {
        // TODO: add l10n
        // showFeedbackMessage(response.error.code + ": " + response.error.message, MessageTypesEnum.DANGER);
        if (!isEmpty(errorCallBack)) {
            errorCallBack(response.data);
        }
        throw response.error.code + ": " + response.error.message +
        " / Details: " + response.error.data;
    }
    if (!isEmpty(callBack)) {
        callBack(response);
    }
}

/* exported getFromRest */
function getFromRest(url, callBack) {
    $.ajax({
        url: url,
        cache: false,
        success: function (response) {
            errorHandler(response, callBack);
        }
    });
}

/* exported createRule */
function createRule() {
    let ruleString = "if ";
    let inp = $('.selectpicker.inputVar').children(':selected').map(function () {
        return $(this).text();
    }).get();
    let inpVal = $('.selectpicker.inputVal').children(':selected').map(function () {
        return $(this).text();
    }).get();
    for (let i = 0; i < inp.length; i++) {
        if (i > 0) {
            ruleString += ' and ';
        }
        ruleString += inp[i] + " is " + inpVal[i];
    }
    $('#ruleContent').val(ruleString);
}

/* exported fillSelect */
function fillSelect(selectElement, values, selectedVal) {
    $(selectElement).html("");
    $.each(values, function (key, value) {
        $(selectElement).append($("<option />").val(value.id).text(value.name));
    });
    $(selectElement).children().each(function () {
        if ($(this).text() == selectedVal) {
            $(this).prop('selected', true);
        }
    });
}

function fillFuzzyTerms(variablesElement, fuzzyTermsElement, termVal) {
    getFromRest("/rule/getFuzzyTerms/" + $(variablesElement).val(), function (fuzzyTerms) {
        let fuzzyTermsData = [];
        $.each(fuzzyTerms, function (key, value) {
            fuzzyTermsData.push({
                id: value.id,
                name: value.description
            });
        });
        fillSelect(fuzzyTermsElement, fuzzyTermsData, termVal);
        $(fuzzyTermsElement).selectpicker("refresh");
        $(fuzzyTermsElement).trigger("change");
    });
}

function fillVariables(projectId, variablesElement, variableVal) {
    getFromRest("/rule/getVariables/" + projectId, function (variables) {
        let variablesData = [];
        $.each(variables, function (key, value) {
            variablesData.push({
                id: value.id,
                name: value.name
            });
        });
        fillSelect(variablesElement, variablesData, variableVal);
        $(variablesElement).selectpicker("refresh");
        $(variablesElement).trigger("change");
    });
}

function variableValueChanged(variablesElement, fuzzyTermsElement) {
    fillFuzzyTerms(variablesElement, fuzzyTermsElement);
}

function fuzzyTermsValueChanged() {
    createRule();
}

function addVariableSelect(projectId, variableVal) {
    let variablesElement = $("<select class='selectpicker inputVar m-2' data-live-search='true data-width='70%'></select>");
    fillVariables(projectId, variablesElement, variableVal);
    return variablesElement;
}

function addFuzzyTermsSelect(variablesElement, termVal) {
    let fuzzyTermsElement = $("<select class='selectpicker inputVal m-2' data-live-search='true data-width='70%'></select>");
    if ($(variablesElement).val()) {
        fillFuzzyTerms(variablesElement, fuzzyTermsElement, termVal);
    }
    return fuzzyTermsElement;
}

function removeAntecedent(buttonElement) {
    $(buttonElement).parent().remove();
    fuzzyTermsValueChanged();
}

function addAntecedent(parentElement, projectId, variableVal, termVal) {
    let rowElement = $("<div class='row'></div>");
    if ($(parentElement).find('.row').length) {
        $(rowElement).append("<label class='col col-md-1 m-2'>И</label>");
    } else {
        $(rowElement).append("<label class='col col-md-1 m-2'> </label>");
    }
    let variablesElement = addVariableSelect(projectId, variableVal);
    let fuzzyTermsElement = addFuzzyTermsSelect(variablesElement, termVal);
    $(variablesElement).on("change", function () {
        variableValueChanged(variablesElement, fuzzyTermsElement)
    });
    $(fuzzyTermsElement).on("change", function () {
        fuzzyTermsValueChanged()
    });
    $(rowElement).append(variablesElement);
    $(rowElement).append("<label class='col col-md-1  m-2'>есть</label>");
    $(rowElement).append(fuzzyTermsElement);
    if ($(parentElement).find('.row').length) {
        $(rowElement).append("<a href='#' class='btn btn-outline-dark m-2' onclick='removeAntecedent($(this))'>-</a>");
    }
    $(parentElement).append(rowElement);
}

function addAntecedentFromRule(parentElement, projectId, ruleContent) {
    let antecedentComponents = getAntecedentComponents(getAntecedent(ruleContent));
    for (let i = 0; i < antecedentComponents.length; i++) {
        var a = antecedentComponents[i];
        addAntecedent(parentElement, projectId, getVariable(a), getVariableValue(a));
    }
}